/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle;


import android.content.Context;
import android.graphics.Color;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

import androidx.annotation.ColorInt;
import androidx.annotation.ColorRes;
import androidx.annotation.FloatRange;
import androidx.annotation.IntRange;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;

import com.compdfkit.core.annotation.CPDFBorderStyle;
import com.compdfkit.core.annotation.CPDFLineAnnotation;
import com.compdfkit.core.annotation.CPDFStampAnnotation;
import com.compdfkit.core.annotation.CPDFTextAttribute;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotationType;

import java.util.ArrayList;
import java.util.List;

public class CAnnotStyle {

    private CAnnotationType type;

    private int color = Color.BLACK;

    private int colorOpacity = 255;

    private float borderWidth = 10;

    private float eraserWidth = 10;

    private int lineColor = Color.BLACK;

    private int lineColorOpacity = 255;

    private int fillColor = Color.BLACK;

    private int fillColorOpacity = 255;

    private CPDFBorderStyle borderStyle = new CPDFBorderStyle(CPDFBorderStyle.Style.Border_Solid, 10, new float[]{8.0F, 9F});

    private CPDFLineAnnotation.LineType startLineType = CPDFLineAnnotation.LineType.LINETYPE_NONE;

    private CPDFLineAnnotation.LineType endLineType = CPDFLineAnnotation.LineType.LINETYPE_NONE;

    private boolean fontBold;

    private boolean fontItalic;

    private int textColor = Color.BLACK;

    private int textColorOpacity = 255;

    private int fontSize;

    private CPDFTextAttribute.FontNameHelper.FontType fontType;

    private String imagePath;

    private String textStampContent;

    private String textStampDate;

    private CPDFStampAnnotation.TextStampShape textStampShape;

    private CPDFStampAnnotation.TextStampColor textStampColor;

    private List<OnAnnotStyleChangeListener> styleChangeListenerList = new ArrayList<>();

    public CAnnotStyle(CAnnotationType type) {
        this.type = type;
    }


    public CAnnotationType getType() {
        return type;
    }

    public int getColor() {
        return color;
    }

    public void setColor(@ColorInt int color) {
        boolean update = color != this.color;
        this.color = color;
        Log.e("CAnnotStyle", "setColor(" + color + ", update:" + update + ")");
        updateColorListener(this.color, update);
    }

    public int getColorOpacity() {
        return colorOpacity;
    }

    public void setColorOpacity(@IntRange(from = 0, to = 255) int colorOpacity) {
        boolean update = colorOpacity != this.colorOpacity;
        this.colorOpacity = colorOpacity;
        Log.e("CAnnotStyle", "setColorOpacity(" + colorOpacity + ", update:" + update + ")");
        updateColorOpacity(this.colorOpacity, update);
    }

    public float getBorderWidth() {
        return borderWidth;
    }

    public void setBorderWidth(@FloatRange(from = 0.0) float borderWidth) {
        boolean update = borderWidth != this.borderWidth;
        this.borderWidth = borderWidth;
        Log.e("CAnnotStyle", "setBorderWidth(" + borderWidth + ", update:" + update + ")");
        updateBorderWidth(this.borderWidth, update);
    }

    public void setEraserWidth(float eraserWidth) {
        this.eraserWidth = eraserWidth;
    }

    public float getEraserWidth() {
        return eraserWidth;
    }

    public int getLineColor() {
        return lineColor;
    }

    public void setLineColor(@ColorInt int lineColor) {
        boolean update = lineColor != this.lineColor;
        this.lineColor = lineColor;
        Log.e("CAnnotStyle", "setLineColor(" + lineColor + ", update:" + update + ")");
        updateLineColor(lineColor, update);
    }

    public int getLineColorOpacity() {
        return lineColorOpacity;
    }

    public void setLineColorOpacity(@IntRange(from = 0, to = 255) int lineColorOpacity) {
        boolean update = lineColorOpacity != this.lineColorOpacity;
        this.lineColorOpacity = lineColorOpacity;
        Log.e("CAnnotStyle", "setLineColorOpacity(" + lineColorOpacity + ", update:" + update + ")");
        updateLineColorOpacity(lineColorOpacity, update);
    }

    public int getFillColor() {
        return fillColor;
    }

    public void setFillColor(@ColorInt int fillColor) {
        boolean update = fillColor != this.fillColor;
        this.fillColor = fillColor;
        Log.e("CAnnotStyle", "setFillColor(" + fillColor + ", update:" + update + ")");
        updateFillColor(fillColor, update);
    }


    public int getFillColorOpacity() {
        return fillColorOpacity;
    }

    public void setFillColorOpacity(@IntRange(from = 0, to = 255) int fillColorOpacity) {
        boolean update = fillColorOpacity != this.fillColorOpacity;
        this.fillColorOpacity = fillColorOpacity;
        Log.e("CAnnotStyle", "setFillColorOpacity(" + fillColor + ", update:" + update + ")");
        updateFillColorOpacity(fillColorOpacity, update);
    }

    public void setBorderStyle(CPDFBorderStyle borderStyle) {
        boolean update = borderStyle.equals(this.borderStyle);
        this.borderStyle = borderStyle;
        Log.e("CAnnotStyle", "setBorderStyle(" + borderStyle + ", update:" + update + ")");
        updateBorderStyle(borderStyle, update);
    }

    public CPDFBorderStyle getBorderStyle() {
        return borderStyle;
    }

    public CPDFLineAnnotation.LineType getStartLineType() {
        return startLineType;
    }

    public void setStartLineType(CPDFLineAnnotation.LineType startLineType) {
        this.startLineType = startLineType;
    }

    public CPDFLineAnnotation.LineType getEndLineType() {
        return endLineType;
    }

    public void setEndLineType(CPDFLineAnnotation.LineType endLineType) {
        this.endLineType = endLineType;
    }

    public boolean isFontBold() {
        return fontBold;
    }

    public void setFontBold(boolean fontBold) {
        this.fontBold = fontBold;
    }

    public boolean isFontItalic() {
        return fontItalic;
    }

    public void setFontItalic(boolean fontItalic) {
        this.fontItalic = fontItalic;
    }

    public int getTextColor() {
        return textColor;
    }

    public void setTextColor(@ColorInt int textColor) {
        this.textColor = textColor;
    }

    public void setTextColor(Context context, @ColorRes int colorRes) {
        this.textColor = ContextCompat.getColor(context, colorRes);
    }

    public int getTextColorOpacity() {
        return textColorOpacity;
    }

    public void setTextColorOpacity(@IntRange(from = 0, to = 255) int textColorOpacity) {
        this.textColorOpacity = textColorOpacity;
    }

    public int getFontSize() {
        return fontSize;
    }

    public void setFontSize(@IntRange(from = 0) int fontSize) {
        this.fontSize = fontSize;
    }

    public CPDFTextAttribute.FontNameHelper.FontType getFontType() {
        return fontType;
    }

    public void setFontType(CPDFTextAttribute.FontNameHelper.FontType fontType) {
        this.fontType = fontType;
    }

    public String getImagePath() {
        return imagePath;
    }

    public void setImagePath(String imagePath) {
        this.imagePath = imagePath;
    }

    public String getTextStampContent() {
        return textStampContent;
    }

    public void setTextStampContent(String textStampContent) {
        this.textStampContent = textStampContent;
    }

    public String getTextStampDate() {
        return textStampDate;
    }

    public void setTextStampDate(String textStampDate) {
        this.textStampDate = textStampDate;
    }

    public CPDFStampAnnotation.TextStampShape getTextStampShape() {
        return textStampShape;
    }

    public void setTextStampShape(CPDFStampAnnotation.TextStampShape textStampShape) {
        this.textStampShape = textStampShape;
    }

    public CPDFStampAnnotation.TextStampColor getTextStampColor() {
        return textStampColor;
    }

    public void setTextStampColor(CPDFStampAnnotation.TextStampColor textStampColor) {
        this.textStampColor = textStampColor;
    }

    @NonNull
    @Override
    public String toString() {
        return "CAnnotStyle{" +
                "type:" + type + "," +
                "color" + color + "," +
                "colorOpacity" + colorOpacity + "," +
                "borderWidth" + borderWidth + "," +
                "lineColor" + lineColor + "," +
                "lineColorOpacity" + lineColorOpacity + "," +
                "fillColor" + fillColor + "," +
                "fillColorOpacity" + fillColorOpacity + "," +
                "borderStyle" + (borderStyle != null ? borderStyle.toString() : "empty") + "," +
                "startLineType" + (startLineType != null ? startLineType.name() : "empty") + "," +
                "endLineType" + (endLineType != null ? endLineType.name() : "empty") + "," +
                "fontBold" + fontBold + "," +
                "fontItalic" + fontItalic + "," +
                "textColor" + textColor + "," +
                "textColorOpacity" + textColorOpacity + "," +
                "fontSize" + fontSize + "," +
                "fontType" + (fontType != null ? fontType.name() : "empty") + "," +
                "imagePath" + imagePath + "," +
                "textStampContent" + textStampContent + "," +
                "textStampDate" + textStampDate + "," +
                "textStampShape" + (textStampShape != null ? textStampShape.name() : "empty") + "," +
                "textStampColor" + (textStampColor != null ? textStampColor.name() : "empty") + ",";
    }

    @Override
    public int hashCode() {
        int result = this.borderWidth != 0.0F ? Float.floatToIntBits(this.borderWidth) : 0;
        result = 31 * result + this.color;
        result = 31 * result + (this.colorOpacity != 0.0F ? Float.floatToIntBits(this.colorOpacity) : 0);
        result = 31 * result + this.lineColor;
        result = 31 * result + (this.lineColorOpacity != 0.0F ? Float.floatToIntBits(this.lineColorOpacity) : 0);
        result = 31 * result + this.fillColor;
        result = 31 * result + (this.fillColorOpacity != 0.0F ? Float.floatToIntBits(this.fillColorOpacity) : 0);
        result = 31 * result + (this.borderStyle != null ? this.borderStyle.hashCode() : 0);
        result = 31 * result + (this.startLineType != null ? this.startLineType.hashCode() : 0);
        result = 31 * result + (this.endLineType != null ? this.endLineType.hashCode() : 0);
        result = 31 * result + this.textColor;
        result = 31 * result + (this.textColorOpacity != 0.0F ? Float.floatToIntBits(this.textColorOpacity) : 0);
        result = 31 * result + (this.fontSize != 0.0F ? Float.floatToIntBits(this.fontSize) : 0);
        result = 31 * result + (this.fontType != null ? this.fontType.hashCode() : 0);
        result = 31 * result + (this.imagePath != null ? this.imagePath.hashCode() : 0);
        result = 31 * result + (this.textStampContent != null ? this.textStampContent.hashCode() : 0);
        result = 31 * result + (this.textStampDate != null ? this.textStampDate.hashCode() : 0);
        result = 31 * result + (this.type != null ? type.hashCode() : 0);
        result = 31 * result + (this.textStampShape != null ? this.textStampShape.hashCode() : 0);
        result = 31 * result + (this.textStampColor != null ? this.textStampColor.hashCode() : 0);
        return result;
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj instanceof CAnnotStyle) {
            return obj.toString().equals(this.toString());
        } else {
            return false;
        }
    }

    public int getAnnotTypeTitleResId() {
        int titleResId = 0;
        switch (type) {
            case TEXT:
                titleResId = R.string.tools_annot_note;
                break;
            case HIGHLIGHT:
                titleResId = R.string.tools_annot_highlight;
                break;
            case STRIKEOUT:
                titleResId = R.string.tools_annot_strikeout;
                break;
            case UNDERLINE:
                titleResId = R.string.tools_annot_underline;
                break;
            case SQUIGGLY:
                titleResId = R.string.tools_annot_squiggly;
                break;
            case INK:
                titleResId = R.string.tools_annot_ink;
                break;
            case SQUARE:
                titleResId = R.string.tools_annot_square;
                break;
            case CIRCLE:
                titleResId = R.string.tools_annot_circle;
                break;
            case LINE:
                titleResId = R.string.tools_annot_line;
                break;
            case ARROW:
                titleResId = R.string.tools_annot_arrow;
                break;
            default:
                titleResId = R.string.tools_menu_pdfview;
                break;
        }
        return titleResId;
    }


    private void updateColorListener(@ColorInt int color, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotColor(color);
            }
        }
    }

    private void updateColorOpacity(int colorOpacity, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotColorOpacity(colorOpacity);
            }
        }
    }

    private void updateBorderWidth(float borderWidth, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotBorderWidth(borderWidth);
            }
        }
    }

    private void updateLineColor(int color, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotLineColor(color);
            }
        }
    }

    private void updateLineColorOpacity(int opacity, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotLineColorOpacity(opacity);
            }
        }
    }

    private void updateFillColor(int color, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotFillColor(color);
            }
        }
    }

    private void updateFillColorOpacity(int opacity, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotFillColorOpacity(opacity);
            }
        }
    }

    private void updateBorderStyle(CPDFBorderStyle style, boolean update) {
        if (styleChangeListenerList != null && update) {
            for (OnAnnotStyleChangeListener onAnnotStyleChangeListener : styleChangeListenerList) {
                onAnnotStyleChangeListener.onChangeAnnotBorderStyle(style);
            }
        }
    }

    public void addStyleChangeListener(OnAnnotStyleChangeListener styleChangeListener) {
        this.styleChangeListenerList.add(styleChangeListener);
    }

    public void addStyleChangeListener(List<OnAnnotStyleChangeListener> styleChangeListener) {
        this.styleChangeListenerList.addAll(styleChangeListener);
    }

    public void removeStyleChangeListener(OnAnnotStyleChangeListener styleChangeListener) {
        this.styleChangeListenerList.remove(styleChangeListener);
    }


    public interface OnAnnotStyleChangeListener {

        void onChangeAnnotColor(@ColorInt int color);

        void onChangeAnnotColorOpacity(int opacity);

        void onChangeAnnotBorderWidth(float borderWidth);

        void onChangeAnnotLineColor(@ColorInt int color);

        void onChangeAnnotLineColorOpacity(int opacity);

        void onChangeAnnotFillColor(@ColorInt int color);

        void onChangeAnnotFillColorOpacity(int opacity);

        void onChangeAnnotBorderStyle(CPDFBorderStyle style);
    }

}
