/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.manager;

import androidx.annotation.ColorInt;
import androidx.annotation.IntRange;

import com.compdfkit.core.annotation.CPDFBorderStyle;
import com.compdfkit.tools.annotation.pdfproperties.CAnnotStyleDialogFragment;
import com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotationType;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.ui.attribute.CPDFAnnotAttribute;
import com.compdfkit.ui.attribute.CPDFCircleAttr;
import com.compdfkit.ui.attribute.CPDFHighlightAttr;
import com.compdfkit.ui.attribute.CPDFInkAttr;
import com.compdfkit.ui.attribute.CPDFLineAttr;
import com.compdfkit.ui.attribute.CPDFSquareAttr;
import com.compdfkit.ui.attribute.CPDFSquigglyAttr;
import com.compdfkit.ui.attribute.CPDFStrikeoutAttr;
import com.compdfkit.ui.attribute.CPDFUnderlineAttr;
import com.compdfkit.ui.attribute.IAttributeCallback;
import com.compdfkit.ui.attribute.form.CPDFTextfieldAttr;

import java.util.LinkedHashSet;

public class CAnnotStyleManager implements CAnnotStyle.OnAnnotStyleChangeListener {

    public void apply(CPDFViewCtrl pdfView, CAnnotStyle style, boolean onStore) {
        LinkedHashSet<CAnnotStyle> linkedHashSet = new LinkedHashSet<>();
        linkedHashSet.add(style);
        apply(pdfView, linkedHashSet, onStore);
    }

    public void apply(CPDFViewCtrl pdfView, LinkedHashSet<CAnnotStyle> styleSet, boolean onStore) {
        CPDFAnnotAttribute attribute = pdfView.getCPdfReaderView().getReaderAttribute().getAnnotAttribute();
        for (CAnnotStyle params : styleSet) {
            if (params == null) {
                continue;
            }
            IAttributeCallback callback = null;
            switch (params.getType()) {
                case TEXT:
                    CPDFTextfieldAttr textfieldAttr = attribute.getTextfieldAttr();
                    textfieldAttr.setBgColor(params.getColor());

                    break;
                case HIGHLIGHT:
                    CPDFHighlightAttr highlightAttr = attribute.getHighlightAttr();
                    highlightAttr.setColor(params.getColor());
                    highlightAttr.setAlpha(params.getColorOpacity());
                    callback = highlightAttr;
                    break;

                case UNDERLINE:
                    CPDFUnderlineAttr underlineAttr = attribute.getUnderlineAttr();
                    underlineAttr.setColor(params.getColor());
                    underlineAttr.setAlpha(params.getColorOpacity());
                    callback = underlineAttr;
                    break;

                case SQUIGGLY:
                    CPDFSquigglyAttr squigglyAttr = attribute.getSquigglyAttr();
                    squigglyAttr.setColor(params.getColor());
                    squigglyAttr.setAlpha(params.getColorOpacity());
                    callback = squigglyAttr;
                    break;

                case STRIKEOUT:
                    CPDFStrikeoutAttr strikeoutAttr = attribute.getStrikeoutAttr();
                    strikeoutAttr.setColor(params.getColor());
                    strikeoutAttr.setAlpha(params.getColorOpacity());
                    callback = strikeoutAttr;
                    break;
                case INK:
                    CPDFInkAttr inkAttr = attribute.getInkAttr();
                    inkAttr.setColor(params.getColor());
                    inkAttr.setAlpha(params.getColorOpacity());
                    inkAttr.setBorderWidth(params.getBorderWidth());
                    inkAttr.setEraseWidth(params.getEraserWidth());
                    break;
                case SQUARE:
                    CPDFSquareAttr squareAttr = attribute.getSquareAttr();
                    squareAttr.setBgColor(params.getFillColor());
                    squareAttr.setBgAlpha(params.getFillColorOpacity());
                    squareAttr.setLineColor(params.getLineColor());
                    squareAttr.setLineAlpha(params.getLineColorOpacity());
                    squareAttr.setBorderWidth(params.getBorderWidth());
                    squareAttr.setBorderStyle(params.getBorderStyle());
                    break;
                case CIRCLE:
                    CPDFCircleAttr circleAttr = attribute.getCircleAttr();
                    circleAttr.setBgColor(params.getFillColor());
                    circleAttr.setBgAlpha(params.getFillColorOpacity());
                    circleAttr.setLineColor(params.getLineColor());
                    circleAttr.setLineAlpha(params.getLineColorOpacity());
                    circleAttr.setBorderWidth(params.getBorderWidth());
                    circleAttr.setBorderStyle(params.getBorderStyle());
                    break;
                case LINE:
                case ARROW:
                    CPDFLineAttr lineAttr = attribute.getLineAttr();
                    lineAttr.setLineColor(params.getLineColor());
                    lineAttr.setLineAlpha(params.getLineColorOpacity());
                    lineAttr.setFillColor(params.getFillColor());
                    lineAttr.setFillAlpha(params.getFillColorOpacity());
                    lineAttr.setBorderWidth(params.getBorderWidth());
                    lineAttr.setBorderStyle(params.getBorderStyle());
                    lineAttr.setLineType(params.getStartLineType(), params.getEndLineType());
                    break;
                default:
                    break;
            }
            if (onStore) {
                if (callback != null) {
                    callback.onstore();
                }
            }
        }
    }

    private CAnnotStyleDialogFragment annotStyleDialogFragment;

    private CPDFViewCtrl pdfView;

    public void bindAnnotStyleFragment(CPDFViewCtrl pdfView, CAnnotStyleDialogFragment styleDialogFragment) {
        this.pdfView = pdfView;
        annotStyleDialogFragment = styleDialogFragment;
        annotStyleDialogFragment.addAnnotStyleChangeListener(this);
    }

    @Override
    public void onChangeAnnotColor(int color) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotColorOpacity(int opacity) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotBorderWidth(float borderWidth) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotLineColor(int color) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotLineColorOpacity(int opacity) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotFillColor(int color) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotFillColorOpacity(int opacity) {
        updateAnnot();
    }

    @Override
    public void onChangeAnnotBorderStyle(CPDFBorderStyle style) {
        updateAnnot();
    }

    private void updateAnnot() {
        if (pdfView != null && annotStyleDialogFragment != null) {
            CAnnotStyle style = annotStyleDialogFragment.getStyle();
            apply(pdfView, style, false);
        }
    }

    public CAnnotStyle getAnnotStyle(CPDFViewCtrl pdfView, CAnnotationType type) {
        CAnnotStyle style = new CAnnotStyle(type);
        CPDFAnnotAttribute attribute = pdfView.getCPdfReaderView().getReaderAttribute().getAnnotAttribute();
        switch (type) {
            case TEXT:
                CPDFTextfieldAttr textfieldAttr = attribute.getTextfieldAttr();
                style.setColor(textfieldAttr.getBgColor());
                break;
            case HIGHLIGHT:
                CPDFHighlightAttr highlightAttr = attribute.getHighlightAttr();
                style.setColor(highlightAttr.getColor());
                style.setColorOpacity(highlightAttr.getAlpha());
                break;

            case UNDERLINE:
                CPDFUnderlineAttr underlineAttr = attribute.getUnderlineAttr();
                style.setColor(underlineAttr.getColor());
                style.setColorOpacity(underlineAttr.getAlpha());
                break;

            case SQUIGGLY:
                CPDFSquigglyAttr squigglyAttr = attribute.getSquigglyAttr();
                style.setColor(squigglyAttr.getColor());
                style.setColorOpacity(squigglyAttr.getAlpha());
                break;

            case STRIKEOUT:
                CPDFStrikeoutAttr strikeoutAttr = attribute.getStrikeoutAttr();
                style.setColor(strikeoutAttr.getColor());
                style.setColorOpacity(strikeoutAttr.getAlpha());
                break;
            case INK:
                CPDFInkAttr inkAttr = attribute.getInkAttr();
                style.setColor(inkAttr.getColor());
                style.setColorOpacity(inkAttr.getAlpha());
                style.setBorderWidth(inkAttr.getBorderWidth());
            case SQUARE:
                CPDFSquareAttr squareAttr = attribute.getSquareAttr();
                style.setFillColor(squareAttr.getBgColor());
                style.setFillColorOpacity(squareAttr.getBgAlpha());
                style.setLineColor(squareAttr.getLineColor());
                style.setLineColorOpacity(squareAttr.getLineAlpha());
                style.setBorderWidth(squareAttr.getBorderWidth());
                style.setBorderStyle(squareAttr.getBorderStyle());
                break;
            case CIRCLE:
                CPDFCircleAttr circleAttr = attribute.getCircleAttr();
                style.setFillColor(circleAttr.getBgColor());
                style.setFillColorOpacity(circleAttr.getBgAlpha());
                style.setLineColor(circleAttr.getLineColor());
                style.setLineColorOpacity(circleAttr.getLineAlpha());
                style.setBorderWidth(circleAttr.getBorderWidth());
                style.setBorderStyle(circleAttr.getBorderStyle());
                break;
            case LINE:
            case ARROW:
                CPDFLineAttr lineAttr = attribute.getLineAttr();
                style.setLineColor(lineAttr.getLineColor());
                style.setLineColorOpacity(lineAttr.getLineAlpha());
                style.setFillColor(lineAttr.getFillColor());
                style.setFillColorOpacity(lineAttr.getFillAlpha());
                style.setBorderStyle(lineAttr.getBorderStyle());
                style.setBorderWidth(lineAttr.getBorderWidth());
                style.setStartLineType(lineAttr.getHeadType());
                style.setEndLineType(lineAttr.getTailType());

                break;
            default:
                break;
        }
        return style;
    }


    public static class Builder {
        private LinkedHashSet<CAnnotStyle> attrSet = new LinkedHashSet();

        private CPDFViewCtrl pdfView;

        public Builder(CPDFViewCtrl pdfView) {
            this.pdfView = pdfView;
        }

        public Builder setColor(CAnnotationType type, @ColorInt int color) {
            CAnnotStyle style = new CAnnotStyle(type);
            style.setColor(color);
            attrSet.add(style);
            return this;
        }

        public Builder setMarkup(CAnnotationType type, @ColorInt int color, @IntRange(from = 0, to = 255) int colorAlpha) {
            CAnnotStyle style = new CAnnotStyle(type);
            style.setColor(color);
            style.setColorOpacity(colorAlpha);
            attrSet.add(style);
            return this;
        }


        public Builder setInkAttribute(@ColorInt int color, @IntRange(from = 0, to = 255) int colorAlpha, int borderWidth) {
            CAnnotStyle style = new CAnnotStyle(CAnnotationType.INK);
            style.setColor(color);
            style.setColorOpacity(colorAlpha);
            style.setBorderWidth(borderWidth);
            attrSet.add(style);
            return this;
        }

        public Builder setEraserWidth(int borderWidth) {
            CAnnotStyle style = new CAnnotStyle(CAnnotationType.INK);
            style.setEraserWidth(borderWidth);
            attrSet.add(style);
            return this;
        }

        public Builder setShape(CAnnotationType type,
                                @ColorInt int lineColor,
                                @IntRange(from = 0, to = 255) int lineColorOpacity,
                                @ColorInt int fillColor,
                                @IntRange(from = 0, to = 255) int fillColorAlpha,
                                float borderWidth,
                                CPDFBorderStyle borderStyle) {
            CAnnotStyle style = new CAnnotStyle(type);
            style.setLineColor(lineColor);
            style.setLineColorOpacity(lineColorOpacity);
            style.setFillColor(fillColor);
            style.setFillColorOpacity(fillColorAlpha);
            style.setBorderWidth(borderWidth);
            if (borderStyle == null) {
                borderStyle = new CPDFBorderStyle();
                borderStyle.setBorderWidth(borderWidth);
                borderStyle.setDashArr(new float[]{8.0F, 0F});
            }

            style.setBorderStyle(borderStyle);
            attrSet.add(style);
            return this;
        }


        private CAnnotStyleManager create() {
            return new CAnnotStyleManager();
        }

        public void clear() {
            attrSet.clear();
        }

        public void apply() {
            create().apply(pdfView, attrSet, false);
        }

        public void onStore() {
            CAnnotStyleManager annotationAttr = create();
            annotationAttr.apply(pdfView, attrSet, true);
        }

        public void init() {
            onStore();
        }
    }
}
