/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.annotation.pdfproperties.pdfshape;


import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.core.annotation.CPDFBorderStyle;
import com.compdfkit.tools.R;
import com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotationType;
import com.compdfkit.tools.common.views.pdfproperties.basic.CBasicPropertiesFragment;
import com.compdfkit.tools.common.views.pdfproperties.colorlist.CColorListView;
import com.compdfkit.tools.common.views.pdfproperties.colorpicker.widget.ColorPickerView;
import com.compdfkit.tools.common.views.pdfproperties.preview.CStylePreviewView;
import com.compdfkit.tools.common.views.pdfproperties.sliderbar.CBorderWidthSliderBar;
import com.compdfkit.tools.common.views.pdfproperties.sliderbar.CSliderBar;

public class CShapeStyleFragment extends CBasicPropertiesFragment {

    private CColorListView borderColorListView;

    private CColorListView fillColorListView;

    private CSliderBar opacitySliderBar;

    private CStylePreviewView stylePreviewView;

    private CBorderWidthSliderBar borderWidthSliderBar;

    private CSliderBar dashedSliderBar;

    private ConstraintLayout clStartLineType;

    private ConstraintLayout clTailLineType;

    public static CShapeStyleFragment newInstance() {
        return new CShapeStyleFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_properties_shape_style_fragment, container, false);
        borderColorListView = rootView.findViewById(R.id.border_color_list_view);
        fillColorListView = rootView.findViewById(R.id.fill_color_list_view);
        stylePreviewView = rootView.findViewById(R.id.style_preview);
        opacitySliderBar = rootView.findViewById(R.id.slider_bar);
        borderWidthSliderBar = rootView.findViewById(R.id.slider_bar_border_width);
        dashedSliderBar = rootView.findViewById(R.id.dashed_slider_bar);
        clStartLineType = rootView.findViewById(R.id.cl_start_line_type);
        clTailLineType = rootView.findViewById(R.id.cl_tail_line_type);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        CAnnotStyle cAnnotStyle = viewModel.getStyle();
        if (cAnnotStyle != null) {
            stylePreviewView.setAnnotType(cAnnotStyle.getType());
            stylePreviewView.setColor(cAnnotStyle.getFillColor());
            stylePreviewView.setColorOpacity(cAnnotStyle.getFillColorOpacity());
            stylePreviewView.setBorderWidth((int) cAnnotStyle.getBorderWidth());
            stylePreviewView.setBorderColor(cAnnotStyle.getLineColor());
            stylePreviewView.setBorderColorOpacity(cAnnotStyle.getLineColorOpacity());
            stylePreviewView.setDashedSpaceWidth((int) cAnnotStyle.getBorderStyle().getDashArr()[1]);

            borderColorListView.setSelectColor(cAnnotStyle.getLineColor());
            fillColorListView.setSelectColor(cAnnotStyle.getFillColor());
            borderWidthSliderBar.setProgress((int) cAnnotStyle.getBorderWidth());

            opacitySliderBar.setProgress(cAnnotStyle.getLineColorOpacity());
            dashedSliderBar.setProgress((int) cAnnotStyle.getBorderStyle().getDashArr()[1]);

            if (cAnnotStyle.getType() == CAnnotationType.ARROW || cAnnotStyle.getType() == CAnnotationType.LINE){
                fillColorListView.setVisibility(View.GONE);
                borderColorListView.setTitle(R.string.tools_color);
                clStartLineType.setVisibility(View.VISIBLE);
                clTailLineType.setVisibility(View.VISIBLE);
            }
        }

        viewModel.addStyleChangeListener(this);

        opacitySliderBar.setChangeListener((opacity, percentageValue) -> updateColorOpacity(opacity));
        borderColorListView.setOnColorSelectListener(this::updateBorderColor);
        fillColorListView.setOnColorSelectListener(this::updateFillColor);
        borderWidthSliderBar.setChangeListener((progress, percentageValue) -> {
            if (viewModel != null) {
                viewModel.getStyle().setBorderWidth(progress);
            }
        });
        dashedSliderBar.setChangeListener((progress, percentageValue) -> {
            if (viewModel != null) {
                CPDFBorderStyle style = viewModel.getStyle().getBorderStyle();
                if (style == null) {
                    style = new CPDFBorderStyle();
                }
                if (progress == 0) {
                    style.setStyle(CPDFBorderStyle.Style.Border_Solid);
                    style.setDashArr(new float[]{8.0F, progress});
                } else {
                    style.setStyle(CPDFBorderStyle.Style.Border_Dashed);
                    style.setDashArr(new float[]{8.0F, progress});
                }
                viewModel.getStyle().setBorderStyle(style);
            }
        });

        borderColorListView.setColorPickerClickListener(() -> {
            CAnnotStyle style = viewModel.getStyle();
            showColorPickerFragment(style.getLineColor(), style.getLineColorOpacity(), new ColorPickerView.COnColorChangeListener() {
                @Override
                public void color(int color) {
                    updateBorderColor(color);
                }

                @Override
                public void opacity(int opacity) {
                    updateColorOpacity(opacity);
                }
            });
        });
        fillColorListView.setColorPickerClickListener(() -> {
            CAnnotStyle style = viewModel.getStyle();
            showColorPickerFragment(style.getFillColor(), style.getFillColorOpacity(), new ColorPickerView.COnColorChangeListener() {
                @Override
                public void color(int color) {
                    updateFillColor(color);
                }

                @Override
                public void opacity(int opacity) {
                    updateColorOpacity(opacity);
                }
            });
        });
    }

    private void updateBorderColor(int color) {
        if (viewModel != null) {
            CAnnotStyle style = viewModel.getStyle();
            if (style.getLineColor() == Color.TRANSPARENT) {
                style.setLineColorOpacity(opacitySliderBar.getProgress());
            }
            viewModel.getStyle().setLineColor(color);
        }
    }

    private void updateFillColor(int fillColor) {
        if (viewModel != null) {
            CAnnotStyle style = viewModel.getStyle();
            if (style.getFillColor() == Color.TRANSPARENT) {
                style.setFillColorOpacity(opacitySliderBar.getProgress());
            }
            style.setFillColor(fillColor);
        }
    }

    private void updateColorOpacity(int opacity) {
        if (viewModel != null) {
            CAnnotStyle style = viewModel.getStyle();
            if (style.getFillColor() != Color.TRANSPARENT) {
                style.setFillColorOpacity(opacity);
            }
            if (style.getLineColor() != Color.TRANSPARENT) {
                style.setLineColorOpacity(opacity);
            }
        }
    }

    @Override
    public void onChangeAnnotFillColor(int color) {
        if (stylePreviewView != null) {
            stylePreviewView.setColor(color);
        }
        if (!isOnResume) {
            if (fillColorListView != null) {
                fillColorListView.setSelectColor(color);
            }
        }
    }

    @Override
    public void onChangeAnnotFillColorOpacity(int opacity) {
        if (stylePreviewView != null) {
            stylePreviewView.setColorOpacity(opacity);
        }
        if (!isOnResume) {
            if (opacitySliderBar != null) {
                opacitySliderBar.setProgress(opacity);
            }
        }
    }

    @Override
    public void onChangeAnnotBorderWidth(float borderWidth) {
        if (stylePreviewView != null) {
            stylePreviewView.setBorderWidth((int) borderWidth);
        }
    }

    @Override
    public void onChangeAnnotLineColor(int color) {
        if (stylePreviewView != null) {
            stylePreviewView.setBorderColor(color);
        }
        if (!isOnResume) {
            if (borderColorListView != null) {
                borderColorListView.setSelectColor(color);
            }
        }
    }

    @Override
    public void onChangeAnnotLineColorOpacity(int opacity) {
        if (stylePreviewView != null) {
            stylePreviewView.setBorderColorOpacity(opacity);
        }
        if (!isOnResume) {
            if (opacitySliderBar != null) {
                opacitySliderBar.setProgress(opacity);
            }
        }
    }

    @Override
    public void onChangeAnnotBorderStyle(CPDFBorderStyle style) {
        super.onChangeAnnotBorderStyle(style);
        if (stylePreviewView != null) {
            stylePreviewView.setDashedSpaceWidth((int) style.getDashArr()[1]);
        }
    }
}
