/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views;


import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.PopupWindow;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.appcompat.widget.PopupMenu;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.widget.PopupMenuCompat;
import androidx.core.widget.PopupWindowCompat;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.utils.window.CPDFToolbarPopupMenu;
import com.compdfkit.tools.viewer.pdfdisplaysettings.CPDFDisplaySettingDialogFragment;
import com.compdfkit.tools.viewer.pdfinfo.CPDFDocumentInfoDialogFragment;

import java.util.ArrayList;
import java.util.List;


/**
 * pdf ui tool bar <br/>
 * ︳------------------------------------------------︳<br/>
 * ︳　　　　　　　　title　　　　　icon1  icon2 icon3   ︳<br/>
 * ︳------------------------------------------------︳<br/>
 * icon1: searchIcon <br/>
 * icon2: boTaIcon <br/>
 * icon3: moreIcon <br/>
 * <p/>
 * use samples:<br/>
 * com.compdfkit.tools.common.utils.view.CPDFToolBar <br/>
 * android:id="@+id/pdf_tool_bar" <br/>
 * android:layout_width="match_parent" <br/>
 * android:layout_height="?android:attr/actionBarSize" <br/>
 * app:tools_toolbar_title="@string/viewer_toolbar_title" <br/>
 * app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_search_icon="@drawable/xxx"/> <br/>
 * <p/>
 * custom attrs: <br/>
 * app:tools_toolbar_title="@string/xxx" <br/>
 * app:tools_toolbar_search_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 * <p/>
 * btn click listener
 * searchBtnClick:
 *
 * @see CPDFToolBar#setSearchBtnClickListener(OnClickListener)
 * <p>
 * botaBtnClick:
 * @see CPDFToolBar#setBoTaBtnClickListener(OnClickListener)
 */
public class CPDFToolBar extends FrameLayout implements CPDFToolbarPopupMenu.OnPreviewModeChangeListener {

    private AppCompatTextView tvToolBarTitle;

    private AppCompatImageView ivToolBarSearchBtn;

    private AppCompatImageView ivToolBarBoTaBtn;

    private AppCompatImageView ivToolBarMoreBtn;

    private AppCompatImageView ivToolBarThumbnail;

    private ConstraintLayout clMainToolbar;

    private AppCompatImageView ivTitleArrow;

    private LinearLayout llTitle;

    private CPDFToolbarPopupMenu popupMenu;

    private OnMoreMenuListener moreMenuListener;

    private CPDFToolbarPopupMenu.OnPreviewModeChangeListener changeListener;

    public CPDFToolBar(@NonNull Context context) {
        this(context, null);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initToolBar(context, attrs);
    }

    private void initToolBar(Context context, @Nullable AttributeSet attrs) {
        LayoutInflater.from(getContext()).inflate(R.layout.tools_cpdf_tool_bar, this);
        clMainToolbar = findViewById(R.id.cl_pdf_root_tool_bar);
        tvToolBarTitle = findViewById(R.id.tv_tool_bar_title);
        ivToolBarMoreBtn = findViewById(R.id.iv_tool_bar_more);
        ivToolBarSearchBtn = findViewById(R.id.iv_tool_bar_search);
        ivToolBarBoTaBtn = findViewById(R.id.iv_tool_bar_bota);
        ivToolBarThumbnail = findViewById(R.id.iv_tool_bar_thumbnail);
        llTitle = findViewById(R.id.ll_title);
        ivTitleArrow = findViewById(R.id.iv_down_arrow);
        popupMenu = new CPDFToolbarPopupMenu(getContext(), llTitle);
        popupMenu.setMenuChangeListener(this);
        llTitle.setOnClickListener(v -> popupMenu.show());
        ivToolBarMoreBtn.setOnClickListener(v -> {
            //Show the PDF settings dixalog fragment
            PopupMenu popupMenu = new PopupMenu(getContext(), v);
            popupMenu.inflate(R.menu.tools_more_menu);
            popupMenu.setOnMenuItemClickListener(item -> {
                if (item.getItemId() == R.id.menu_id_document_info) {
                    if (moreMenuListener != null) {
                        moreMenuListener.menu(MoreMenu.DocumentInfo);
                    }
                } else if (item.getItemId() == R.id.menu_id_reader_settings) {
                    if (moreMenuListener != null) {
                        moreMenuListener.menu(MoreMenu.Settings);
                    }
                }
                return true;
            });
            popupMenu.show();
        });
        initAttributes(context, attrs);
    }

    private void initAttributes(Context context, @Nullable AttributeSet attrs) {
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.CPDFToolBar);
        if (typedArray != null) {
            String toolBarTitle = typedArray.getString(R.styleable.CPDFToolBar_tools_toolbar_title);
            typedArray.getResourceId(R.styleable.CPDFToolBar_tools_toolbar_search_icon, -1);
            if (!TextUtils.isEmpty(toolBarTitle)) {
                tvToolBarTitle.setText(toolBarTitle);
            }
            Drawable searchIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_search_icon, R.drawable.tools_ic_search);
            if (searchIconDrawable != null) {
                ivToolBarSearchBtn.setImageDrawable(searchIconDrawable);
            }
            Drawable boTaIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_bota_icon, R.drawable.tools_ic_outline);
            if (boTaIconDrawable != null) {
                ivToolBarBoTaBtn.setImageDrawable(boTaIconDrawable);
            }
            Drawable moreIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_more_icon, R.drawable.tools_ic_more);
            if (moreIconDrawable != null) {
                ivToolBarMoreBtn.setImageDrawable(moreIconDrawable);
            }
            CViewUtils.applyViewBackground(this);
            typedArray.recycle();
        }
    }

    public void setSearchBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarSearchBtn.setOnClickListener(clickListener);
    }

    public void setBoTaBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarBoTaBtn.setOnClickListener(clickListener);
    }

    public void setThumbnailBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarThumbnail.setOnClickListener(clickListener);
    }

    public void setPreviewModeChangeListener(CPDFToolbarPopupMenu.OnPreviewModeChangeListener changeListener) {
        this.changeListener = changeListener;
    }

    @Override
    public void change(CPDFToolbarPopupMenu.PreviewMode mode, String title) {
        tvToolBarTitle.setText(title);
        if (changeListener != null) {
            changeListener.change(mode, title);
        }
    }

    public void setMoreMenuListener(OnMoreMenuListener moreMenuListener) {
        this.moreMenuListener = moreMenuListener;
    }

    public void addMode(CPDFToolbarPopupMenu.PreviewMode previewMode) {
        popupMenu.addMode(previewMode);
        tvToolBarTitle.setText(popupMenu.getTitleByMode(previewMode));
        ivTitleArrow.setVisibility(popupMenu.getMenuCount() >1 ? VISIBLE : GONE);
    }

    public CPDFToolbarPopupMenu.PreviewMode getMode(){
        return popupMenu != null ? popupMenu.getMode() : CPDFToolbarPopupMenu.PreviewMode.PDFView;
    }

    public enum MoreMenu {
        DocumentInfo,

        Settings
    }

    public interface OnMoreMenuListener {
        void menu(MoreMenu menu);
    }
}
