/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfannotationbar;


import android.content.Context;
import android.util.AttributeSet;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.tools.R;
import com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.CBasicOnStyleChangeListener;
import com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.manager.CAnnotStyleManager;
import com.compdfkit.tools.common.views.pdfannotationbar.adapter.CPDFAnnotationToolListAdapter;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotToolBean;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotationType;
import com.compdfkit.tools.common.views.pdfannotationbar.data.CAnnotationToolDatas;
import com.compdfkit.tools.annotation.pdfproperties.CAnnotStyleDialogFragment;
import com.compdfkit.tools.annotation.pdfproperties.pdfannotstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.ui.proxy.attach.IInkDrawCallback;

public class CAnnotationToolbar extends FrameLayout {

    private RecyclerView rvAnnotationList;

    private AppCompatImageView ivSetting;

    private AppCompatImageView ivUndo;

    private AppCompatImageView ivRedo;

    private CPDFAnnotationToolListAdapter toolListAdapter;

    private CPDFViewCtrl pdfView;

    private FragmentManager fragmentManager;

    private OnAnnotationChangeListener annotationChangeListener;

    public CAnnotationToolbar(@NonNull Context context) {
        this(context, null);
    }

    public CAnnotationToolbar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CAnnotationToolbar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    private void initView(Context context) {
        inflate(context, R.layout.tools_annot_tool_bar, this);
        rvAnnotationList = findViewById(R.id.rv_annotation);
        ivSetting = findViewById(R.id.iv_annotation_attr_settings);
        ivUndo = findViewById(R.id.iv_annotation_attr_undo);
        ivRedo = findViewById(R.id.iv_annotation_attr_redo);
        toolListAdapter = new CPDFAnnotationToolListAdapter();
        rvAnnotationList.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        rvAnnotationList.setAdapter(toolListAdapter);
        initListener();
    }

    private void initListener() {
        ivSetting.setEnabled(false);
        ivSetting.setOnClickListener(v -> {
            CAnnotStyleManager manager = new CAnnotStyleManager();
            CAnnotStyle style = manager.getAnnotStyle(pdfView, toolListAdapter.getCurrentAnnotType());

            CAnnotStyleDialogFragment dialogFragment = CAnnotStyleDialogFragment.newInstance(style);
            manager.bindAnnotStyleFragment(pdfView, dialogFragment);
            dialogFragment.addAnnotStyleChangeListener(new CBasicOnStyleChangeListener() {
                @Override
                public void onChangeAnnotColor(int color) {
                    super.onChangeAnnotColor(color);
                    toolListAdapter.updateItemColor(toolListAdapter.getCurrentAnnotType(), color);
                }
            });
            dialogFragment.show(fragmentManager, "annotStyleDialogFragment");
        });

        ivUndo.setOnClickListener(v -> {
            undo();
        });
        ivRedo.setOnClickListener(v -> {
            redo();
        });
        toolListAdapter.setOnItemClickListener((adapter, view, position) -> {
            CAnnotToolBean bean = adapter.list.get(position);
            toolListAdapter.selectItem(position);
            switchAnnotationType(bean);
        });
    }

    public void initWithPDFView(CPDFViewCtrl pdfView) {
        this.pdfView = pdfView;
        toolListAdapter.setList(CAnnotationToolDatas.getAnnotationList(pdfView));
        this.pdfView.addOnPDFFocusedTypeChangeListener(type -> {
            if (type == CPDFAnnotation.Type.UNKNOWN) {
                if (toolListAdapter.hasSelectAnnotType()) {
                    toolListAdapter.selectByType(CAnnotationType.UNKNOWN);
                    ivSetting.setEnabled(toolListAdapter.hasSelectAnnotType());
                }
            }
        });
    }

    public void setFragmentManager(FragmentManager fragmentManager) {
        this.fragmentManager = fragmentManager;
    }

    private void switchAnnotationType(CAnnotToolBean bean) {
        ivSetting.setEnabled(toolListAdapter.hasSelectAnnotType());
        if (!bean.isSelect()) {
            pdfView.resetAnnotationType();
            pdfView.getCPdfReaderView().getInkDrawHelper().onClean();
            if (annotationChangeListener != null) {
                annotationChangeListener.change(CAnnotationType.UNKNOWN);
            }
            return;
        }
        if (bean.getType() != CAnnotationType.INK) {
            pdfView.getCPdfReaderView().getInkDrawHelper().onClean();
            pdfView.getCPdfReaderView().getInkDrawHelper().setMode(IInkDrawCallback.Mode.DRAW);
        }
        switch (bean.getType()) {
            case TEXT:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.TEXT);
                break;
            case INK:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.INK);
                pdfView.getCPdfReaderView().getInkDrawHelper().setEffect(IInkDrawCallback.Effect.NORMAL);
                break;
            case ARROW:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.LINE);
                break;
            case SIGNATURE:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.STAMP);
                break;
            case PIC:
                break;
            default:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.valueOf(bean.getType().name()));
                break;
        }
        if (annotationChangeListener != null) {
            annotationChangeListener.change(bean.getType());
        }
    }

    private void undo() {
        switch (toolListAdapter.getCurrentAnnotType()) {
            case INK:
                if (pdfView != null) {
                    pdfView.getCPdfReaderView().getInkDrawHelper().onUndo();
                }
                break;
            default:
                break;
        }
    }

    private void redo() {
        switch (toolListAdapter.getCurrentAnnotType()) {
            case INK:
                if (pdfView != null) {
                    pdfView.getCPdfReaderView().getInkDrawHelper().onRedo();
                }
                break;
            default:
                break;
        }
    }

    public void setAnnotationChangeListener(OnAnnotationChangeListener annotationChangeListener) {
        this.annotationChangeListener = annotationChangeListener;
    }

    public interface OnAnnotationChangeListener {
        void change(CAnnotationType type);
    }
}
