/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfproperties.preview;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.ColorInt;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotationType;


public class CStylePreviewView extends FrameLayout {

    private CAnnotationType annotationType = CAnnotationType.TEXT;

    private CBasicAnnotPreviewView previewView;

    public CStylePreviewView(@NonNull Context context) {
        this(context, null);
    }

    public CStylePreviewView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CStylePreviewView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initAttr(context, attrs);
    }

    private void initAttr(Context context, AttributeSet attrs) {
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.CStylePreviewView);
        if (typedArray != null) {
            int annotEnumId = typedArray.getInt(R.styleable.CStylePreviewView_tools_annot_type, 0);
            setAnnotType(getCAnnotTypeById(annotEnumId));
            typedArray.recycle();
        }
    }

    public void setAnnotType(CAnnotationType type) {
        this.annotationType = type;
        updatePreview();
    }

    public void setColor(@ColorInt int color) {
        if (previewView != null) {
            previewView.setColor(color);
        }
    }

    public void setColorOpacity(int colorOpacity) {
        if (previewView != null) {
            previewView.setColorOpacity(colorOpacity);
        }
    }

    public void setBorderWidth(int width) {
        if (previewView != null) {
            previewView.setBorderWidth(width);
        }
    }

    public void setBorderColor(@ColorInt int color) {
        if (previewView != null) {
            previewView.setBorderColor(color);
        }
    }

    public void setBorderColorOpacity(int opacity) {
        if (previewView != null) {
            previewView.setBorderColorOpacity(opacity);
        }
    }

    public void setDashedSpaceWidth(int dashedSpace) {
        if (previewView != null) {
            previewView.setDashedGsp(dashedSpace);
        }
    }

    private void updatePreview() {
        removeAllViews();
        previewView = null;
        switch (annotationType) {
            case TEXT:
                previewView = new CAnnotNotePreviewView(getContext());
                break;
            case HIGHLIGHT:
            case UNDERLINE:
            case SQUIGGLY:
            case STRIKEOUT:
                CAnnotMarkupPreviewView view = new CAnnotMarkupPreviewView(getContext());
                view.setMarkupType(annotationType);
                previewView = view;
                break;
            case INK:
                previewView = new CAnnotInkPreviewView(getContext());
                break;
            case SQUARE:
                previewView = new CAnnotShapePreviewView(getContext());
                previewView.setShapeType(CAnnotShapePreviewView.CShapeView.ShapeType.SQUARE);
                break;
            case CIRCLE:
                previewView = new CAnnotShapePreviewView(getContext());
                previewView.setShapeType(CAnnotShapePreviewView.CShapeView.ShapeType.CIRCLE);
                break;
            case LINE:
                previewView = new CAnnotShapePreviewView(getContext());
                previewView.setShapeType(CAnnotShapePreviewView.CShapeView.ShapeType.LINE);
                break;
            case ARROW:
                previewView = new CAnnotShapePreviewView(getContext());
                previewView.setShapeType(CAnnotShapePreviewView.CShapeView.ShapeType.ARROW);
                break;
            default:
                break;
        }
        if (previewView != null) {
            addView(previewView, new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        }
    }

    private CAnnotationType getCAnnotTypeById(int attrEnumId) {
        switch (attrEnumId) {
            case 0:
                return CAnnotationType.TEXT;
            case 1:
                return CAnnotationType.HIGHLIGHT;
            case 2:
                return CAnnotationType.UNDERLINE;
            case 3:
                return CAnnotationType.SQUIGGLY;
            case 4:
                return CAnnotationType.STRIKEOUT;
            case 5:
                return CAnnotationType.INK;
            case 6:
                return CAnnotationType.CIRCLE;
            case 7:
                return CAnnotationType.SQUARE;
            case 8:
                return CAnnotationType.ARROW;
            case 9:
                return CAnnotationType.LINE;
            case 10:
                return CAnnotationType.FREE_TEXT;
            case 11:
                return CAnnotationType.SIGNATURE;
            default:
                return CAnnotationType.UNKNOWN;
        }
    }

}
