/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.viewer.pdfdisplaysettings;


import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.RadioGroup;
import android.widget.Switch;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatRadioButton;
import androidx.core.content.ContextCompat;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.dialog.CDialogFragmentUtil;
import com.compdfkit.tools.common.views.CToolBar;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.ui.reader.CPDFReaderView;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.List;

public class CPDFDisplaySettingDialogFragment extends BottomSheetDialogFragment
        implements RadioGroup.OnCheckedChangeListener, CompoundButton.OnCheckedChangeListener, View.OnClickListener {

    /**
     * Bright mode, white color
     */
    private static final int READER_LIGHT_MODE = 0xFFFFFFFF;

    /**
     * Dark mode, dark gray color
     */
    private static final int READER_DARK_MODE = 0xFF333333;

    /**
     * Sepia mode, a color similar to the brown color of paper
     */
    private static final int READER_SEPIA_MODE = 0xFFFFEFBE;

    /**
     * Eye protection mode, light green color
     */
    private static final int READER_RESEDA_MODE = 0XFFCDE6D0;

    private final List<View> customThemesViews = new ArrayList<>();

    private CToolBar toolBar;

    private RadioGroup readerModeRadioGroup;

    private AppCompatRadioButton rbSinglePage;

    private AppCompatRadioButton rbDoublePage;

    private AppCompatRadioButton rbCoverDoublePage;

    private Switch swIsContinue;

    private Switch swIsVertical;

    private Switch swIsCropMode;

    private RadioGroup readerBgRadioGroup;

    private AppCompatRadioButton rbLightMode;

    private AppCompatRadioButton rbDarkMode;

    private AppCompatRadioButton rbSepiaMode;

    private AppCompatRadioButton rbResedaMode;

    private AppCompatImageView ivCustomTheme1;

    private AppCompatImageView ivCustomTheme2;

    private AppCompatImageView ivCustomTheme3;

    private AppCompatImageView ivCustomTheme4;

    private CPDFViewCtrl pdfView;

    public static CPDFDisplaySettingDialogFragment newInstance() {
        return new CPDFDisplaySettingDialogFragment();
    }

    public void initWithPDFView(CPDFViewCtrl pdfView) {
        this.pdfView = pdfView;
    }

    @Override
    public void onStart() {
        super.onStart();
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        CDialogFragmentUtil.setBottomSheetDialogFragmentFullScreen(getDialog(), behavior);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_display_settings_dialog_fragment, container, false);
        toolBar = rootView.findViewById(R.id.tool_bar);
        toolBar.setBackBtnClickListener(v -> dismiss());
        readerModeRadioGroup = rootView.findViewById(R.id.radio_group_reader_mode);
        rbSinglePage = rootView.findViewById(R.id.r_btn_single_page);
        rbDoublePage = rootView.findViewById(R.id.r_btn_double_page);
        rbCoverDoublePage = rootView.findViewById(R.id.r_btn_cover_double_page);
        swIsContinue = rootView.findViewById(R.id.sw_is_continue);
        swIsVertical = rootView.findViewById(R.id.sw_is_vertical);
        swIsCropMode = rootView.findViewById(R.id.sw_is_crop);
        readerBgRadioGroup = rootView.findViewById(R.id.radio_group_reader_bg);
        rbLightMode = rootView.findViewById(R.id.r_btn_light_mode);
        rbDarkMode = rootView.findViewById(R.id.r_btn_dark_mode);
        rbSepiaMode = rootView.findViewById(R.id.r_btn_sepia_mode);
        rbResedaMode = rootView.findViewById(R.id.r_btn_reseda_mode);
        ivCustomTheme1 = rootView.findViewById(R.id.iv_custom_theme_1);
        ivCustomTheme2 = rootView.findViewById(R.id.iv_custom_theme_2);
        ivCustomTheme3 = rootView.findViewById(R.id.iv_custom_theme_3);
        ivCustomTheme4 = rootView.findViewById(R.id.iv_custom_theme_4);

        customThemesViews.add(ivCustomTheme1);
        customThemesViews.add(ivCustomTheme2);
        customThemesViews.add(ivCustomTheme3);
        customThemesViews.add(ivCustomTheme4);

        ivCustomTheme1.setOnClickListener(this);
        ivCustomTheme2.setOnClickListener(this);
        ivCustomTheme3.setOnClickListener(this);
        ivCustomTheme4.setOnClickListener(this);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (pdfView != null) {
            com.compdfkit.ui.reader.CPDFReaderView readerView = pdfView.getCPdfReaderView();
            if (readerView.isDoublePageMode()) {
                if (readerView.isCoverPageMode()) {
                    rbCoverDoublePage.setChecked(true);
                } else {
                    rbDoublePage.setChecked(true);
                }
            } else {
                rbSinglePage.setChecked(true);
            }
            swIsContinue.setChecked(readerView.isContinueMode());
            swIsVertical.setChecked(readerView.isVerticalMode());
            swIsCropMode.setChecked(readerView.isCropMode());
            initThemes(readerView);
        }

        swIsContinue.setOnCheckedChangeListener(this);
        swIsVertical.setOnCheckedChangeListener(this);
        swIsCropMode.setOnCheckedChangeListener(this);

        readerModeRadioGroup.setOnCheckedChangeListener(this);
        readerBgRadioGroup.setOnCheckedChangeListener(this);
    }

    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
        AppCompatRadioButton radioButton = group.findViewById(checkedId);
        if (radioButton == null || !radioButton.isChecked()){
            return;
        }
        if (checkedId == R.id.r_btn_light_mode) {
            setReaderBackgroundColor(READER_LIGHT_MODE);
            selectCustomTheme(null);
        } else if (checkedId == R.id.r_btn_dark_mode) {
            setReaderBackgroundColor(READER_DARK_MODE);
            selectCustomTheme(null);
        } else if (checkedId == R.id.r_btn_sepia_mode) {
            setReaderBackgroundColor(READER_SEPIA_MODE);
            selectCustomTheme(null);
        } else if (checkedId == R.id.r_btn_reseda_mode) {
            setReaderBackgroundColor(READER_RESEDA_MODE);
            selectCustomTheme(null);
        } else if (checkedId == R.id.r_btn_single_page) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setDoublePageMode(false);
                pdfView.getCPdfReaderView().setCoverPageMode(false);
            }
        } else if (checkedId == R.id.r_btn_double_page) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setDoublePageMode(true);
                pdfView.getCPdfReaderView().setCoverPageMode(false);
            }
        } else if (checkedId == R.id.r_btn_cover_double_page) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setDoublePageMode(true);
                pdfView.getCPdfReaderView().setCoverPageMode(true);
            }
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (buttonView.getId() == R.id.sw_is_vertical) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setVerticalMode(isChecked);
            }
        } else if (buttonView.getId() == R.id.sw_is_continue) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setContinueMode(isChecked);
            }
        } else if (buttonView.getId() == R.id.sw_is_crop) {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().setCropMode(isChecked);
            }
        }
    }

    @Override
    public void onClick(View v) {
        selectCustomTheme(v);
        if (getContext() == null) {
            return;
        }
        readerBgRadioGroup.clearCheck();
        if (v.getId() == R.id.iv_custom_theme_1) {
            setReaderBackgroundColor(ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_1));
        } else if (v.getId() == R.id.iv_custom_theme_2) {
            setReaderBackgroundColor(ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_2));
        } else if (v.getId() == R.id.iv_custom_theme_3) {
            setReaderBackgroundColor(ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_3));
        } else if (v.getId() == R.id.iv_custom_theme_4) {
            setReaderBackgroundColor(ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_4));
        }
    }


    private void initThemes(CPDFReaderView readerView) {
        if (getContext() == null) {
            return;
        }
        int bgColor = readerView.getReadBackgroundColor();
        if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_themes_light)) {
            rbLightMode.setChecked(true);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_themes_dark)) {
            rbDarkMode.setChecked(true);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_themes_sepia)) {
            rbSepiaMode.setChecked(true);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_themes_reseda)) {
            rbResedaMode.setChecked(true);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_1)) {
            selectCustomTheme(ivCustomTheme1);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_2)) {
            selectCustomTheme(ivCustomTheme2);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_3)) {
            selectCustomTheme(ivCustomTheme3);
        } else if (bgColor == ContextCompat.getColor(getContext(), R.color.tools_custom_theme_color_4)) {
            selectCustomTheme(ivCustomTheme4);
        } else {

        }
    }

    private void selectCustomTheme(@Nullable View customThemeView) {
        for (View view : customThemesViews) {
            if (view == customThemeView) {
                view.setSelected(true);
            } else {
                view.setSelected(false);
            }
        }
    }

    private void setReaderBackgroundColor(int color) {
        if (pdfView != null) {
            pdfView.getCPdfReaderView().setReadBackgroundColor(color);
        }
    }
}
